# -*- coding: utf-8 -*-
"""
An rfc5424/rfc5425 syslog server implementation
Copyright © 2011 Evax Software <contact@evax.fr>
"""
import socket,os,sys
from datetime import datetime

from loggerglue.rfc5424 import DEFAULT_PRIVAL,SyslogEntry
from loggerglue.emitter import UNIXSyslogEmitter

class Logger(object):
    """
    Convenience class to log RFC5424 messages to the
    local syslog daemon (by default) or a remote
    syslog receiver.
    """

    def __init__(self, emitter=None, hostname=None, app_name=None, procid=None):
        """
        **Arguments**
            *emitter*
                Emitter object to send syslog messages, default to Unix socket /dev/log
                
            *hostname*
                Hostname to send with log messages, defaults to system hostname
                
            *app_name*
                Application name to send with log messages, defaults to application name
                
            *procid*
                Process ID to send with log messages, default to current process ID
        """
        if hostname is None:
            # Compute host name to submit to syslog
            hostname = socket.gethostname()

        if app_name is None:
            # Compute default app name from name of executable,
            # without extension.
            app_name = os.path.basename(sys.argv[0])
            (app_name, _, _) = app_name.partition(".") 

        if procid is None:
            procid = os.getpid()

        if emitter is None:
            emitter = UNIXSyslogEmitter()

        self.hostname = hostname
        self.app_name = app_name
        self.procid = procid
        self.emitter = emitter

    def log(self, msg=None, msgid=None, structured_data=None, prival=DEFAULT_PRIVAL, 
            timestamp=None):
        """
        Log a message. 

        Example:
        
           >>> logger.log("test", prival=LOG_DEBUG|LOG_MAIL)

        **Arguments**
            *msg*
                Human readable message to log
            *msgid*
                Message identifier
            *structured_data*
                Structured data to attach to log message
            *prival*
                Priority and facility of message (defaults to INFO|USER)
            *timestamp*
                UTC time of log message (default to current time)
        """
        if timestamp is None:
            timestamp = datetime.utcnow()

        msg = SyslogEntry(
                    prival=prival, timestamp=datetime.utcnow(), 
                    hostname=self.hostname, app_name=self.app_name, procid=self.procid, msgid=msgid,
                    structured_data=structured_data, 
                    msg=msg
            )

        self.emitter.emit(msg)

    def close(self):
        """
        Close connection to logger.
        """
        self.emitter.close()




from datetime import datetime, timedelta

def parse_date(ts):
    '''
    Parse syslog date with optional sub-second precision.
    '''
    if '.' in ts:
        return datetime.strptime(ts, "%Y-%m-%dT%H:%M:%S.%f")
    else:
        return datetime.strptime(ts, "%Y-%m-%dT%H:%M:%S")

def parse_timestamp(ts):
    '''
    Parse timestamps in these formats to a datetime structure:
    '2003-10-11T22:14:15.003000Z'
    '2003-10-11T22:14:15.003Z'
    '2003-10-11T22:14:15Z'
    '2003-10-11T22:14:15.003000-07:00'
    '''
    if ts == '-':
        timestamp = None
    elif ts[-1] == 'Z':
        timestamp = parse_date(ts[:-1])
    else:
        timestamp = parse_date(ts[:-6])
        hours = int(ts[-5:-3])
        mins = int(ts[-2:])
        sign = ts[-6] == '-' and -1 or 1
        timestamp += timedelta(seconds=sign*(hours*3600+mins*60))

    return timestamp



"""The syslog constants are replicated here because not all systems have the (binary) syslog module. 
RFC5424 priority values are a combination of a priority and facility code, for example:

    prival = LOG_ALERT | LOG_DAEMON

Priorities
----------

============== =====================
Constant        Description
============== =====================
LOG_EMERG       System is unusable
LOG_ALERT       Action must be taken immediately
LOG_CRIT        Critical conditions
LOG_ERR         Error conditions
LOG_WARNING     Warning conditions
LOG_NOTICE      Normal but significant condition
LOG_INFO        Informational
LOG_DEBUG       Debug-level messages
============== =====================

Facility codes
--------------

=============  ===========================================
Constant        Description
=============  ===========================================
LOG_KERN        Kernel messages
LOG_USER        Random user-level messages
LOG_MAIL        Mail system
LOG_DAEMON      System daemons
LOG_AUTH        Security/authorization messages
LOG_SYSLOG      Messages generated internally by syslogd
LOG_LPR         Line printer subsystem
LOG_NEWS        Network news subsystem
LOG_UUCP        UUCP subsystem
LOG_CRON        Clock daemon
LOG_AUTHPRIV    Security/authorization messages (private)
LOG_LOCAL0      Reserved for local use
LOG_LOCAL1      Reserved for local use
LOG_LOCAL2      Reserved for local use
LOG_LOCAL3      Reserved for local use
LOG_LOCAL4      Reserved for local use
LOG_LOCAL5      Reserved for local use
LOG_LOCAL6      Reserved for local use
LOG_LOCAL7      Reserved for local use
=============  ===========================================
"""

LOG_EMERG     = 0
LOG_ALERT     = 1
LOG_CRIT      = 2
LOG_ERR       = 3
LOG_WARNING   = 4
LOG_NOTICE    = 5
LOG_INFO      = 6
LOG_DEBUG     = 7

LOG_KERN      = 0<<3
LOG_USER      = 1<<3
LOG_MAIL      = 2<<3
LOG_DAEMON    = 3<<3
LOG_AUTH      = 4<<3
LOG_SYSLOG    = 5<<3
LOG_LPR       = 6<<3
LOG_NEWS      = 7<<3
LOG_UUCP      = 8<<3
LOG_CRON      = 9<<3
LOG_AUTHPRIV  = 10<<3
LOG_LOCAL0    = 16<<3
LOG_LOCAL1    = 17<<3
LOG_LOCAL2    = 18<<3
LOG_LOCAL3    = 19<<3
LOG_LOCAL4    = 20<<3
LOG_LOCAL5    = 21<<3
LOG_LOCAL6    = 22<<3
LOG_LOCAL7    = 23<<3




# -*- coding: utf-8 -*-
"""
An rfc5424/rfc5425 syslog server implementation
Copyright © 2011 Evax Software <contact@evax.fr>
"""
import types, socket, ssl

# Default UDP port to send syslog messages
SYSLOG_DEFAULT_PORT             = 514

class SyslogEmitter(object):
    """
    Base class for syslog emitters. 
    
    A syslog emitter provides
    two methods, one to send messages and one to close the connection.
    All emitters are set up such that the connection is re-established
    once when a network issue happens. If the re-connection attempt fails,
    a socket exception is thrown.
    
    Derived classes have specific constructors describing the address
    to send messages to.
    """
    def close(self):
        """
        Closes the socket.
        """
        pass

    def emit(self, msg):
        """
        Emit a log record.
        """
        pass

class UDPSyslogEmitter(SyslogEmitter):
    """
    Syslog emitter through UDP.
    
    Sends syslog messages over UDP. As UDP is an unreliable protocol,
    use of this class is discouraged. The only use-case would be 
    sending messages to a syslog server that does not support TCP.
    """
    def __init__(self, address=('localhost', SYSLOG_DEFAULT_PORT)):
        """Create a Syslog emitter that sends messages through UDP.
        
        **Arguments**
        
            *address* 
                address to send messages to, as `(host,port)` tuple
        """
        self.address = address
        self._connect(address)
        
    def _connect(self, address):
        """(Re-)connect to socket"""
        self.socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)

    def close(self):
        """
        Closes the socket.
        """
        pass

    def emit(self, msg):
        """
        Emit a record.
        """
        self.socket.sendto(str(msg), self.address)

class UNIXSyslogEmitter(SyslogEmitter):
    def __init__(self, address='/dev/log'):
        """Create a Syslog emitter that sends messages through a UNIX
        socket. This is useful for sending messages to a local
        syslog daemon.        
        
        **Arguments**
        
            *address*
                Address to send messages to, as string. Defaults to '/dev/log'.
        """
        self.address = address
        self._connect(address)
        
    def _connect(self, address):
        """(Re-)connect to socket"""
        self.socket = socket.socket(socket.AF_UNIX, socket.SOCK_DGRAM)
        try:
            self.socket.connect(address)
        except socket.error:
            self.socket.close()
            self.socket = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
            self.socket.connect(address)

    def close(self):
        """
        Closes the socket.
        """
        self.socket.close()

    def emit(self, msg):
        """
        Emit a record.
        """
        try:
            self.socket.send(str(msg)+'\000')
        except socket.error:
            self._connect(self.address)
            self.socket.send(str(msg)+'\000')

class TCPSyslogEmitter(SyslogEmitter):
    """
    Syslog emitter that sends messages through a TCP socket. Optionally supports TLS.
    """
    def __init__(self, address=('localhost', SYSLOG_DEFAULT_PORT),\
        octet_based_framing=True, **ssl_args):
        """ 
        **Arguments**
            *address*
                Address to send messages to, specify this as `(host, port)` tuple.

            *octet_based_framing*
                Use RFC5425 octet-based framing instead of line-based framing. Use
                this when sending multiline messages.
                
            *keyfile*, *certfile*, *server_side*, *cert_reqs*, *ssl_version*, *ca_certs*, *ciphers*
                Arguments to pass through to :func:`ssl.wrap_socket`. Providing any of these arguments
                enables TLS.
        """
        self.address = address
        self.octet_based_framing = octet_based_framing
        self.ssl_args = ssl_args
        self._connect(address, ssl_args)
        
    def _connect(self, address, ssl_args):
        """(Re-)connect to socket"""
        self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.socket.connect(address)
        if ssl_args:
            self.socket = ssl.wrap_socket(self.socket, server_side=False,
                                   **ssl_args)

    def close(self):
        """
        Closes the socket.
        """
        try:
            self.socket.close()
        except socket.error:
            pass
        
    def _send(self, msg):
        msg = str(msg)
        if self.octet_based_framing:
            self.socket.send("%i %s" % (len(msg), msg))
        else:
            self.socket.send(msg + '\n')

    def emit(self, msg):
        """
        Emit a record.
        """
        try:
            self._send(msg)
        except socket.error:
            self._connect(self.address, self.ssl_args)
            self._send(msg)




"""
Tests for both the Syslog server and emitter.
"""
import unittest
from loggerglue.emitter import TCPSyslogEmitter
from loggerglue.server import SyslogServer,SyslogHandler
from loggerglue.rfc5424 import StructuredData, SyslogEntry, SDElement
from datetime import datetime
import socket, os, time, threading
from tempfile import NamedTemporaryFile

def create_test_entry(proto):
    hostname = "test.example.com"
    app_name = "app_name"
    y = SyslogEntry(
            prival=165, timestamp=datetime.utcnow(), 
            hostname=hostname, app_name=app_name, procid=os.getpid(), msgid='ID47',
            structured_data=[SDElement('exampleSDID@32473', 
                [('iut','3'),
                ('eventSource','Application'),
                ('eventID','1011'),
                ('eventID','1012')]
                )], 
            msg='An application event log entry through '+proto+'...'
    )
    return y

class Handler(SyslogHandler):
    def handle_entry(self, syslog_entry):
        self.server.entry = syslog_entry

def syslog_server_thread(serv):
    """Handle one request"""
    serv.handle_request()

# SSL test certificate/key
key_data = """
-----BEGIN RSA PRIVATE KEY-----
MIICXAIBAAKBgQC+K1APSGZB9BsZJwUa6L1qIa8eIxxJtZnYknMsf34ZnJzpg8vw
LjjeuFB4OAmIudY7iHDkBIRrOOUemtWf5zOjZIZUZBWXtH5dpoNSkdcU1UnVYFyV
GkS05Sta2sDL0M1Vo0HQTzj2D7OGxQkvoza9/oeLmH5WyFqTWfMySP0lYQIDAQAB
AoGAeHnaI4R7fAcY78tfmZefAFDMr2wQ7MWG7MSDAMrYaYNliS69EbFGyBq+qBrg
N2V8SoT6+AnAlmkaBlSTR9ViSWbA4TZYZt7wrcu/ld20GE4k3K2zua6Pl4bgxG2h
QzmpDQoID7eKPfZiIUch7sCjzxqv4nJNmE3gO+MicX6cf0ECQQDdt+YXJN7Tb3YS
N35cMGA/ZEi7eYYlIux3wEDTm4Mm93EmE1S+BU2J/fm7P5nihctV4ebgwUicE/nG
ahVXEyKZAkEA25Kg49VbV09IrS3iP1gaY3DDWaenfJNWwDlUOymk3KPzc/gbiuza
S50sWZJ//Wi3bTIrJPwcGtkHLAvAUjseCQJAMqbcjdUCgtMn6il7WJxEoLbMVugA
WWONGh51sOIKKFDHLKel6HNVr3yyHLD++t0OAuTE1fvSFrYJjeaWUXoxoQJAPWU5
kZs16CrmIm5jBd1Hu6hrJyWG4oF8T1F4aPaS/5LkXvfwE594xo3TOdSJ7zyZlXHi
uHu6DBPFOp6qjxOyqQJBAK8xkuYCCbZYEiiCt21xS+D7emOxQZ0eZf938u3A3/7p
FYoTjD5y9gK7utBjaQYtR/dIK5HnRQTycxvmdW4o2dQ=
-----END RSA PRIVATE KEY-----
"""

cert_data = """
-----BEGIN CERTIFICATE-----
MIIDEzCCAnwCCQDq8BfSOBmMxjANBgkqhkiG9w0BAQUFADCBzTELMAkGA1UEBhMC
WFgxKjAoBgNVBAgTIVRoZXJlIGlzIG5vIHN1Y2ggdGhpbmcgb3V0c2lkZSBVUzET
MBEGA1UEBxMKRXZlcnl3aGVyZTEOMAwGA1UEChMFT0NPU0ExPDA6BgNVBAsTM09m
ZmljZSBmb3IgQ29tcGxpY2F0aW9uIG9mIE90aGVyd2lzZSBTaW1wbGUgQWZmYWly
czERMA8GA1UEAxMIcGhvc3Bob3IxHDAaBgkqhkiG9w0BCQEWDXJvb3RAcGhvc3Bo
b3IwHhcNMDgwNTMwMTA0NzQ1WhcNMDgwNjI5MTA0NzQ1WjCBzTELMAkGA1UEBhMC
WFgxKjAoBgNVBAgTIVRoZXJlIGlzIG5vIHN1Y2ggdGhpbmcgb3V0c2lkZSBVUzET
MBEGA1UEBxMKRXZlcnl3aGVyZTEOMAwGA1UEChMFT0NPU0ExPDA6BgNVBAsTM09m
ZmljZSBmb3IgQ29tcGxpY2F0aW9uIG9mIE90aGVyd2lzZSBTaW1wbGUgQWZmYWly
czERMA8GA1UEAxMIcGhvc3Bob3IxHDAaBgkqhkiG9w0BCQEWDXJvb3RAcGhvc3Bo
b3IwgZ8wDQYJKoZIhvcNAQEBBQADgY0AMIGJAoGBAL4rUA9IZkH0GxknBRrovWoh
rx4jHEm1mdiScyx/fhmcnOmDy/AuON64UHg4CYi51juIcOQEhGs45R6a1Z/nM6Nk
hlRkFZe0fl2mg1KR1xTVSdVgXJUaRLTlK1rawMvQzVWjQdBPOPYPs4bFCS+jNr3+
h4uYflbIWpNZ8zJI/SVhAgMBAAEwDQYJKoZIhvcNAQEFBQADgYEAZHouYp7yKH0o
b0Z8KRGxu408vAjGFg9nXS0BhK3N+zNy574f3/NFHlNR8nNMcROEKHH6NmXpYnuQ
sRtTT/S7b2ctasdeZHH8V/DNvVRRe55gB3z3AeG+OLvOgRHROgpNYStttYJrarEp
fALEMmG3RMNk2r0FlksHlCQV7L9jp8M=
-----END CERTIFICATE-----
"""


class TestServer(unittest.TestCase):

    def test_tcp(self):
        address = ('127.0.0.1', 5514)
        
        serv = SyslogServer(address, Handler)
        serv.entry = None
        
        thr = threading.Thread(
            target=syslog_server_thread, args=(serv,))
        thr.start()
        tm = TCPSyslogEmitter(address, octet_based_framing=False)
        y = create_test_entry('TCP')
        tm.emit(y)
        tm.close()
        thr.join()
        serv.socket.close()
        
        self.assertEqual(serv.entry.msg, "An application event log entry through TCP...")

    def test_tcps(self):
        address = ('127.0.0.1', 5515)
        
        keyfile = NamedTemporaryFile(delete=False)
        keyfile.write(key_data)
        keyfile.close()
        certfile = NamedTemporaryFile(delete=False)
        certfile.write(cert_data)
        certfile.close()
        serv = SyslogServer(address, Handler, 
            keyfile=keyfile.name,
            certfile=certfile.name)
        serv.entry = None

        thr = threading.Thread(
            target=syslog_server_thread, args=(serv,))
        thr.start()
        tm = TCPSyslogEmitter(address, octet_based_framing=True,
                keyfile=None)
        y = create_test_entry('TCPS')
        tm.emit(y)
        tm.close()
        thr.join()

        serv.socket.close()
        
        os.unlink(keyfile.name)
        os.unlink(certfile.name)

        self.assertEqual(serv.entry.msg, "An application event log entry through TCPS...")

if __name__ == '__main__':
    unittest.main()




# -*- coding: utf-8 -*-
"""
A parser for the Syslog Protocol (RFC5424 - http://tools.ietf.org/search/rfc542)

Copyright © 2011 Evax Software <contact@evax.fr>
"""
from datetime import datetime, timedelta
from pyparsing import Word, Regex, Group, White, Combine, CharsNotIn, \
    ZeroOrMore, OneOrMore, QuotedString, Or, Optional, LineStart, LineEnd, \
    printables, ParseException
from loggerglue.util.MultiDict import OrderedMultiDict
from loggerglue.util.escape_value import escape_param_value, str_or_nil
from loggerglue.util.parse_timestamp import parse_timestamp

# Support SYSLOG_SyslogProtocol23Format which can send an empty APP-NAME.
SUPPORT_MISSING_VALUES = True

# from the RFCs ABNF description
nilvalue = Word("-")
digit = Regex("[0-9]{1}")
nonzero_digit = Regex("[1-9]{1}")
printusascii = printables
sp = White(" ", exact=1)
octet = Regex('[\x00-\xFF]')
utf_8_string = Regex('[\x00-\xFF]*')
BOM = '\xef\xbb\xbf'
bom = Regex(BOM)
msg_utf8 = bom + utf_8_string
msg_any = utf_8_string
msg = Combine(Or([msg_utf8, msg_any])).setResultsName('MSG')
sd_name = CharsNotIn('= ]"', 1, 32)
param_name = sd_name.setResultsName('SD_PARAM_NAME')
param_value = QuotedString(quoteChar='"', escChar='\\', multiline=True)
param_value = param_value.setResultsName('SD_PARAM_VALUE')
sd_id = sd_name.setResultsName('SD_ID')
sd_param = Group(param_name + Regex('=') + param_value)
sd_params = Group(ZeroOrMore(Group(sp+sd_param.setResultsName('SD_PARAM'))))
sd_element = Group('['+sd_id+sd_params.setResultsName('SD_PARAMS')+']')
sd_element = sd_element.setResultsName('SD_ELEMENT')
sd_elements = Group(OneOrMore(sd_element))
structured_data = Or([nilvalue, sd_elements.setResultsName('SD_ELEMENTS')])
structured_data = structured_data.setResultsName('STRUCTURED_DATA')
time_hour = Regex('0[0-9]|1[0-9]|2[0-3]')
time_minute = Regex('[0-5][0-9]')
time_second = time_minute
time_secfrac = Regex('\.[0-9]{1,6}')
time_numoffset = Or([Regex('\+'), Regex('-')]) + \
                 time_hour + ':' + time_minute
time_offset = Or([Regex('Z'), time_numoffset])
partial_time = time_hour + ':' + time_minute + ':' + time_second + \
               Optional(time_secfrac)
full_time = partial_time + time_offset
date_mday = Regex('[0-9]{2}')
date_month = Regex('0[1-9]|1[0-2]')
date_fullyear = Regex('[0-9]{4}')
full_date = date_fullyear + '-' + date_month + '-' + date_mday
timestamp = Combine(Or([nilvalue, full_date + 'T' + full_time]))
timestamp = timestamp.setResultsName('TIMESTAMP')
msgid = Or([nilvalue, CharsNotIn('= ]"', 1, 32)])
if SUPPORT_MISSING_VALUES:
    msgid = Optional(msgid)
msgid = msgid.setResultsName('MSGID')
procid = Or([nilvalue,CharsNotIn('= ]"', 1, 128)])
if SUPPORT_MISSING_VALUES:
    procid = Optional(procid)
procid = procid.setResultsName('PROCID')
app_name = Or([nilvalue, CharsNotIn('= ]"', 1, 48)])
if SUPPORT_MISSING_VALUES:
    app_name = Optional(app_name)
app_name= app_name.setResultsName('APP_NAME')
hostname = Or([nilvalue, CharsNotIn('= ]"', 1, 255)])
if SUPPORT_MISSING_VALUES:
    hostname = Optional(hostname)
hostname = hostname.setResultsName('HOSTNAME')
version = Regex('[1-9][0-9]{0,2}').setResultsName('VERSION')
prival = Regex("[0-9]{1,3}").setResultsName('PRIVAL')
pri = "<" + prival + ">"
header = pri + version + sp + timestamp + sp + hostname + sp + \
         app_name + sp + procid + sp + msgid
syslog_msg = LineStart() + header + structured_data + \
             Optional(sp+msg) + LineEnd()

# Default Prival for new SyslogEntry instances
from constants import LOG_INFO,LOG_USER
DEFAULT_PRIVAL = LOG_INFO|LOG_USER

class Params(object):
    def __init__(self, d):
        for k, v in d.items():
            setattr(self, k, v)

class SDElement(object):
    """
    An SD-ELEMENT consists of a name and parameter name-value pairs.
    """
    def __init__(self, sd_id, sd_params):
        """
        **arguments**
            *sd_id*
                SD-IDs are case-sensitive and uniquely identify the type and purpose
                of the SD-ELEMENT.
            
            *sd_params*
                Key/value pairs attached to this SD-ELEMENT. This can be any iterable
                that yields tuples, a dict or a :class:`~loggerglue.utils.multidict.OrderedMultiDict`
                (An SD-PARAM key may be repeated multiple times inside an SD-ELEMENT)
                
        **attributes**
            *id*
                SD-ID for this structured data element.
                
            *sd_params*
                Key/value pairs attached to this SD-ELEMENT, represented as 
                a multidict.
            
            *params*
                Key/value pairs attached to this SD-ELEMENT, represented as
                a class instance (for convenience, so that parameters can
                be addressed with `elmt.params.origin`). If there are multiple
                values for a key, the *last* element is returned.
                
        """
        self.id = sd_id
        self.sd_params = OrderedMultiDict(sd_params)
        self.params = Params(self.sd_params)
        
    def __str__(self):
        """Convert SDElement to formatted string"""
        rv = ['[', self.id]
        for (k,v) in self.sd_params.allitems():
            rv += [' ',k,'="',escape_param_value(v),'"']
        rv += [']']
        return ''.join(rv)

    @classmethod
    def parse(cls, parsed):
        sd = getattr(parsed, 'STRUCTURED_DATA', None)
        if sd is None or sd == '-':
            return None
        sd_id = parsed.STRUCTURED_DATA.SD_ID
        params = OrderedMultiDict()
        for i in parsed.STRUCTURED_DATA.SD_PARAMS:
            params[i.SD_PARAM.SD_PARAM_NAME] = \
                    i.SD_PARAM.SD_PARAM_VALUE.decode('utf-8')
        return StructuredData(sd_id, params)

class StructuredData(object):
    def __init__(self, elements):
        self.elements = elements
        
    def __str__(self):
        """Convert StructuredData to string"""
        return ''.join([str(e) for e in self.elements])

    @classmethod
    def parse(cls, parsed):
        sd = getattr(parsed, 'STRUCTURED_DATA', None)
        if sd is None or sd == '-':
            return None
        elements = []
        for se in parsed.SD_ELEMENTS:
            sd_id = se.SD_ID
            params = OrderedMultiDict()
            for i in se.SD_PARAMS:
                params[i.SD_PARAM.SD_PARAM_NAME] = \
                        i.SD_PARAM.SD_PARAM_VALUE.decode('utf-8')
            elements.append(SDElement(sd_id, params))
        return StructuredData(elements)

    @classmethod
    def from_str(cls, line):
        """Returns a StructuredData object from a string"""
        try:
            r = structured_data.parseString(line)
            return cls.parse(r)
        except Exception, e:
            print e
            import sys, traceback
            traceback.print_exc(file=sys.stdout)

            return None
    
class SyslogEntry(object):
    """
    A class representing a syslog entry.
    """
    def __init__(self, prival=DEFAULT_PRIVAL, version=1, timestamp=None, 
            hostname=None, app_name=None, procid=None, msgid=None,
            structured_data=None, msg=None):
        """
        **arguments/attributes**
            
            *prival*
                RFC5424 priority values are a combination of a priority and facility, for example `LOG_ALERT | LOG_DAEMON`.
                See :mod:`loggerglue.constants`.
            
            *version*
                Version of syslog entry. There is usually no need to change this.
            
            *timestamp*
                Timestamp (as a datetime object).
            
            *hostname*
                The HOSTNAME field SHOULD contain the hostname and the domain name of the originator.
            
            *app_name*
                The APP-NAME field SHOULD identify the device or application that
                originated the message.  It is a string without further semantics.
                It is intended for filtering messages on a relay or collector.
            
            *procid*
                PROCID is a value that is included in the message, having no
                interoperable meaning, except that a change in the value indicates
                there has been a discontinuity in syslog reporting. 
            
            *msgid*
                The MSGID SHOULD identify the type of message.
            
            *structured_data*
                STRUCTURED-DATA provides a mechanism to express information in a well
                defined, easily parseable and interpretable data format.
            
            *msg*
                The MSG part contains a free-form message that provides information
                about the event.

        """
        self.prival = prival
        self.version = version
        self.timestamp = timestamp
        self.hostname = hostname
        self.app_name = app_name
        self.procid = procid
        self.msgid = msgid
        if structured_data is not None and not isinstance(structured_data, StructuredData):
            structured_data = StructuredData(structured_data)
        self.structured_data = structured_data
        self.msg = msg
    
    @classmethod
    def parse(cls, parsed):
        ts = parse_timestamp(parsed.TIMESTAMP)
        if ts is None:
            # If no timestamp provided, fill in current UTC date and time
            timestamp = datetime.utcnow()
        else:
            timestamp = ts
        attr = {}
        for i in ('prival', 'version', 'hostname', 'app_name',
                  'procid', 'msgid'):
            I = i.upper()
            v = getattr(parsed, I, '-')
            if v in ["", "-"]:
                v = None
            else:
                v = v.decode('utf-8')
            attr[i] = v
        m = getattr(parsed, 'MSG', None)
        if m is not None:
            if m.startswith(BOM):
                msg = m[3:].decode('utf-8')
            else:
                msg = unicode(m)
        else:
            msg = None
        version = int(attr['version'])
        prival = int(attr['prival'])
        structured_data = StructuredData.parse(parsed)
        return cls(
            prival=prival, version=version, timestamp=timestamp, 
            hostname=attr['hostname'], app_name=attr['app_name'], procid=attr['procid'], msgid=attr['msgid'],
            structured_data=structured_data, msg=msg
        )
        
    def __str__(self):
        """Convert SyslogEntry to string"""
        rv = ['<', str(self.prival), '>', str(self.version), ' ']
        if self.timestamp is None:
            rv.append('-')
        else:
            rv.append(self.timestamp.strftime("%Y-%m-%dT%H:%M:%S.%fZ"))
        rv += [' ',
               str_or_nil(self.hostname), ' ', str_or_nil(self.app_name), ' ', str_or_nil(self.procid), ' ',
               str_or_nil(self.msgid),    ' ', str_or_nil(self.structured_data)]
        if self.msg is not None:
            rv += [' ']
            if type(self.msg) is unicode:
                rv += [BOM, self.msg.encode('utf-8')]
            else:
                rv += [self.msg]
        return ''.join(rv)

    @classmethod
    def from_line(cls, line):
        """Returns a parsed SyslogEntry object from a syslog `line`."""
        try:
            r = syslog_msg.parseString(line.strip())
            return cls.parse(r)
        except Exception, e:
            print e
            import sys, traceback
            traceback.print_exc(file=sys.stdout)

            return None




# -*- coding: utf-8 -*-
"""
An rfc5424/rfc5425 syslog server implementation
Copyright © 2011 Evax Software <contact@evax.fr>
"""
import ssl
import SocketServer
from loggerglue.rfc5424 import SyslogEntry

class SyslogHandler(SocketServer.BaseRequestHandler):
    """
    Handler for syslog connections. An instance of this class is created for each incoming connection.
    
    Subclasses must implement `handle_entry` and may implement `handle_error`.
    """
    def setup(self):
        if not self.server.use_tls:
            self.connection = self.request.makefile()

    def finish(self):
        if not self.server.use_tls:
            self.connection.close()

    def handle(self):
        if self.server.use_tls:
            return self.handle_tls()
        while True:
            line = self.connection.readline()
            if not line:
                break
            syslog_entry = SyslogEntry.from_line(line)
            if syslog_entry:
                self.handle_entry(syslog_entry)
            else:
                self.handle_error(line)

    def handle_tls(self):
        buf = ''
        while True:
            r = self.request.recv(1)
            if not r:
                break # EOF
            if r != ' ':
                buf += r
            else:
                try:
                    msg_len = int(buf)
                except:
                    # Protocol error
                    return
                buf = ''
                for i in xrange(msg_len):
                    buf += self.request.recv(1)
                syslog_entry = SyslogEntry.from_line(buf)
                buf = ''
                if syslog_entry is None:
                    self.handle_error(buf)
                self.handle_entry(syslog_entry)

    def handle_entry(self, syslog_entry):
        """Handle an incoming syslog entry. Subclasses must implement this.
        """
        raise NotImplemented('Subclasses must implement this method')

    def handle_error(self, data):
        """Handle an error. Subclasses can implemnt this.
        
        Implementing this method is optional.
        """
        pass

class SyslogServer(SocketServer.TCPServer, SocketServer.ThreadingMixIn):
    """
    TCP Syslog server based on SocketServer.
    """
    allow_reuse_address = True

    _allowed_ssl_args = ('keyfile', 'certfile', 'cert_reqs', 'ssl_version',
                         'ca_certs', 'suppress_ragged_eofs')

    def __init__(self, server_address, RequestHandlerClass,
                 bind_and_activate=True,
                 **ssl_args):
        """
        **arguments**
            *server_address*
                Address to bind to, as a tuple `(host,port)`. Example: `('127.0.0.1',1234)`.
            
            *RequestHandlerClass*
                Class to instantiate for connections. Pass a subclass of 
                :class:`~loggerglue.server.SyslogHandler`.
            
            *bind_and_activate*
                Automatically  call server_bind and server_activate.
            
            *keyfile*, *certfile*, *server_side*, *cert_reqs*, *ssl_version*, *ca_certs*, *ciphers*
                Arguments to pass through to :func:`ssl.wrap_socket`. Providing any of these arguments
                enables TLS.
        """
        self.use_tls = False
        if ssl_args:
            for arg in ssl_args:
                if arg not in self._allowed_ssl_args:
                    raise TypeError('unexpected keyword argument: %s' %arg)
            self.ssl_args = ssl_args
            self.use_tls = True
        SocketServer.TCPServer.__init__(self, server_address,
                                        RequestHandlerClass,
                                        bind_and_activate)

    def get_request(self):
        conn, addr = self.socket.accept()
        if self.use_tls:
            conn = ssl.wrap_socket(conn, server_side=True,
                                   **self.ssl_args)
        return (conn, addr)




import os
from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.txt')).read()
CHANGES = open(os.path.join(here, 'CHANGES.txt')).read()

execfile('loggerglue/version.py')

setup(name='loggerglue',
      version=__version__,
      description='Syslog protocol (rfc5424 and rfc5425) utilities',
      long_description= README + '\n\n' + CHANGES,
      author='Evax Software',
      author_email='contact@evax.fr',
      url='http://www.evax.fr/',
      license='MIT License',
      packages=find_packages(),
      include_package_data=True,
      install_requires=[
          'setuptools',
          'pyparsing',
          ],
      keywords = ['syslog', 'rfc5424', 'rfc5425', 'TLS'],
      classifiers = [
              "Development Status :: 4 - Beta",
              "Intended Audience :: Developers",
              "Intended Audience :: System Administrators",
              "License :: OSI Approved :: MIT License",
              "Programming Language :: Python",
              "Operating System :: OS Independent",
              "Topic :: System :: Logging",
              "Topic :: Internet :: Log Analysis",
              ],
      test_suite = "loggerglue.tests",
      )




import re

escape_re = re.compile(r'([\"\\\]])')

def escape_param_value(s):
    '''
    Escape PARAM-VALUE. Inside PARAM-VALUE, the characters '"' (ABNF %d34), '\' (ABNF %d92),
    and ']' (ABNF %d93) MUST be escaped.
    '''
    return escape_re.sub(r'\\\1', s)

def str_or_nil(s):
    '''
    Return the string value of s, or NILVALUE if s is None.
    '''
    if s is None:
        return '-'
    else:
        return str(s)

        


"""Dictionary-like objects which allow multiple keys

Python dictionaries map a key to a value.  Duplicate keys are not
allowed, and new entries replace old ones with the same key.  Order is
not otherwise preserved, so there's no way to get the items in the
order they were added to a dictionary.

Some types of data is best stored in dictionary-like object which
allow multiple values per key.  Some of these need the input order
strongly preserved, so the items can be retrieved in the same order as
they were added to the dictionary.  That is the OrderedMultiDict.

Others need a weaker ordering guarantee where the order of values for
a given key is preserved but the order between the keys is not.  That
is UnorderedMultiDict.  (Because strong ordering isn't needed, it's
faster delete from an UnorderedMultiDict.)

To create a MultiDict, pass in an object which implements the
'allitems' method and returns a list of (key, value) pairs, or
pass in the list of (key, value) pairs directly.

The two MultiDict classes implement the following dictionary methods
  d["lookup"],
  d["key"] = value
  del d[key]
  d.get("key", default = None)
  d1 == d2, d1 != d2, len(d), iter(d), str(d)
  d.keys(), d.values(), d.items()

The new methods are:
  d.getall(key)
  d.allkeys()
  d.allvalues()
  d.allitems()
  
  >>> import MultiDict
  >>> od = MultiDict.OrderedMultiDict()
  >>> od["Name"] = "Andrew"
  >>> od["Color"] = "BLUE"
  >>> od["Name"] = "Dalke"
  >>> od["Color"] = "Green"
  >>> od[3] = 9
  >>> len(od)
  3
  >>> od["Name"]
  'Dalke'
  >>> od.getall("Name")
  ['Andrew', 'Dalke']
  >>> for k, v in od.allitems():
  ...     print "%r == %r" % (k, v)
  ...
  'Name' == 'Andrew'
  'Color' == 'BLUE'
  'Name' == 'Dalke'
  'Color' == 'Green'
  3 == 9
  >>> del od["Name"]
  >>> len(od)
  2
  >>> for k, v in od.allitems():
  ...     print "%r == %r" % (k, v)
  ...
  'Color' == 'BLUE'
  'Color' == 'Green'
  3 == 9
  >>>

The latest version of this code can be found at
  http://www.dalkescientific.com/Python/
"""
# Written in 2003 by Andrew Dalke, Dalke Scientific Software, LLC.
# This software has been released to the public domain.  No
# copyright is asserted.

from __future__ import generators

# Implementation inheritence -- not asserting a class hierarchy here
#
# If there is a class hierarchy, OrderedMultiDict is a child of
# UnorderedMultiDict because it makes stronger but not different
# guarantees on how the data works, at least data-wise.
# Performance-wise, Ordered has a slower (O(n)) than Unordered (O(1)).
# Convince me otherwise and I'll change.  Besides, hierarchies are
# overrated.
class _BaseMultiDict:
    def __str__(self):
        """shows contents as if this is a dictionary

        If multiple values exist for a given key, use the last
        one added.
        """
        d = {}
        for k in self.data:
            d[k] = self.data[k][-1]
        return str(d)
    def __len__(self):
        """the number of unique keys"""
        return len(self.data)

    def __getitem__(self, key):
        """value for a given key

        If more than one value exists for the key, use one added most recently
        """
        return self.data[key][-1]

    def get(self, key, default = None):
        """value for the given key; default = None if not present
        
        If more than one value exists for the key, use the one added
        most recently.
        """
        return self.data.get(key, [default])[-1]
    
    def __contains__(self, key):
        """check if the key exists"""
        return key in self.data

    def keys(self):
        """unordered list of unique keys"""
        return self.data.keys()

    def values(self):
        """unordered list of values

        If more than one value exists for a given key, use the value
        added most recently.
        """
        return [x[-1] for x in self.data.values()]
    
    def items(self):
        """unordered list of key/value pairs

        If more than one value exists for a given key, use the value
        added most recently.
        """
        return [(k, v[-1]) for k, v in self.data.items()]

    def getall(self, key):
        """Get all values for a given key

        Multiple values are returned in input order.
        If the key does not exists, returns an empty list.
        """
        return self.data[key]

    def __iter__(self):
        """iterate through the list of unique keys"""
        return iter(self.data)

    
class OrderedMultiDict(_BaseMultiDict):
    """Store key/value mappings.

    Acts like a standard dictionary with the following features:
       - duplicate keys are allowed;

       - input order is preserved for all key/value pairs.

    >>> od = OrderedMultiDict([("Food", "Spam"), ("Color", "Blue"),
    ...                        ("Food", "Eggs"), ("Color", "Green")])
    >>> od["Food"]
    'Eggs'
    >>> od.getall("Food")
    ['Spam', 'Eggs']
    >>> list(od.allkeys())
    ['Food', 'Color', 'Food', 'Color']
    >>>

    The order of keys and values(eg, od.allkeys() and od.allitems())
    preserves input order.

    Can also pass in an object to the constructor which has an
    allitems() method that returns a list of key/value pairs.

    """
    def __init__(self, multidict = None):
        self.data = {}
        self.order_data = []
        if multidict is not None:
            if hasattr(multidict, "allitems"):
                multidict = multidict.allitems()
            elif hasattr(multidict, "items"):
                multidict = multidict.items()
            for k, v in multidict:
                self[k] = v
    def __eq__(self, other):
        """Does this OrderedMultiDict have the same contents and order as another?"""
        return self.order_data == other.order_data
    def __ne__(self, other):
        """Does this OrderedMultiDict have different contents or order as another?"""
        return self.order_data != other.order_data
    
    def __repr__(self):
        return "<OrderedMultiDict %s>" % (self.order_data,)

    def __setitem__(self, key, value):
        """Add a new key/value pair

        If the key already exists, replaces the existing value
        so that d[key] is the new value and not the old one.

        To get all values for a given key, use d.getall(key).
        """
        self.order_data.append((key, value))
        self.data.setdefault(key, []).append(value)

    def __delitem__(self, key):
        """Remove all values for the given key"""
        del self.data[key]
        self.order_data[:] = [x for x in self.order_data if x[0] != key]

    def allkeys(self):
        """iterate over all keys in input order"""
        for x in self.order_data:
            yield x[0]
    def allvalues(self):
        """iterate over all values in input order"""
        for x in self.order_data:
            yield x[1]
    def allitems(self):
        """iterate over all key/value pairs in input order"""
        return iter(self.order_data)


    
class UnorderedMultiDict(_BaseMultiDict):
    """Store key/value mappings.

    Acts like a standard dictionary with the following features:
       - duplicate keys are allowed;

       - input order is preserved for all values of a given
           key but not between different keys.

    >>> ud = UnorderedMultiDict([("Food", "Spam"), ("Color", "Blue"),
    ...                          ("Food", "Eggs"), ("Color", "Green")])
    >>> ud["Food"]
    'Eggs'
    >>> ud.getall("Food")
    ['Spam', 'Eggs']
    >>>

    The order of values from a given key (as from ud.getall("Food"))
    is guaranteed but the order between keys (as from od.allkeys()
    and od.allitems()) is not.

    Can also pass in an object to the constructor which has an
    allitems() method that returns a list of key/value pairs.

    """
    def __init__(self, multidict = None):
        self.data = {}
        if multidict is not None:
            if hasattr(multidict, "allitems"):
                multidict = multidict.allitems()
            elif hasattr(multidict, "items"):
                multidict = multidict.items()
            for k, v in multidict:
                self[k] = v

    def __eq__(self, other):
        """Does this UnorderedMultiDict have the same keys, with values in the same order, as another?"""
        return self.data == other.data

    def __ne__(self, other):
        """Does this UnorderedMultiDict NOT have the same keys, with values in the same order, as another?"""
        return self.data != other.data

    def __repr__(self):
        return "<UnorderedMultiDict %s>" % (self.data,)

    def __setitem__(self, key, value):
        """Add a new key/value pair

        If the key already exists, replaces the existing value
        so that d[key] is the new value and not the old one.

        To get all values for a given key, use d.getall(key).
        """
        self.data.setdefault(key, []).append(value)

    def __delitem__(self, key):
        """Remove all values for the given key"""
        del self.data[key]

    def allkeys(self):
        """iterate over all keys in arbitrary order"""
        for k, v in self.data.iteritems():
            for x in v:
                yield k

    def allvalues(self):
        """iterate over all values in arbitrary order"""
        for v in self.data.itervalues():
            for x in v:
                yield x

    def allitems(self):
        """iterate over all key/value pairs, in arbitrary order

        Actually, the keys are iterated in arbitrary order but all
        values for that key are iterated at sequence of addition
        to the UnorderedMultiDict.

        """
        for k, v in self.data.iteritems():
            for x in v:
                yield (k, x)

__test__ = {
    "test_ordered_multidict": """
        >>> od = OrderedMultiDict()
        >>> od["Name"] = "Andrew"
        >>> od["Color"] = "BLUE"
        >>> od["Name"] = "Dalke"
        >>> od["Color"] = "Green"
        >>> od[3] = 9
        >>> len(od)
        3
        >>> len(od.keys())
        3
        >>> len(od.values())
        3
        >>> len(od.items())
        3
        >>> od.keys()
        ['Color', 3, 'Name']
        >>> "Name" in od and "Name" in od.keys() and "Name" in od.allkeys()
        1
        >>> "Color" in od and "Color" in od.keys() and "Color" in od.allkeys()
        1
        >>> 3 in od and 3 in od.keys() and 3 in od.allkeys()
        1
        >>> od == od
        1
        >>> od != OrderedMultiDict()    # line 25
        1
        >>> list(od.allkeys())
        ['Name', 'Color', 'Name', 'Color', 3]
        >>> list(od.allvalues())
        ['Andrew', 'BLUE', 'Dalke', 'Green', 9]
        >>> list(od.allitems())
        [('Name', 'Andrew'), ('Color', 'BLUE'), ('Name', 'Dalke'), ('Color', 'Green'), (3, 9)]
        >>> len(list(od))
        3
        >>> od["invalid"]
        Traceback (most recent call last):
          File "<stdin>", line 1, in ?
          File "MultiDict.py", line 33, in __getitem__
            return self.data[key]
        KeyError: invalid
        >>> od["Color"]
        'Green'
        >>> od.getall("Color")
        ['BLUE', 'Green']
        >>> od2 = OrderedMultiDict(od)
        >>> list(od2.allitems())
        [('Name', 'Andrew'), ('Color', 'BLUE'), ('Name', 'Dalke'), ('Color', 'Green'), (3, 9)]
        >>> od == od2
        1
        >>> od2 == od                  # line 53
        1
        >>> od2 != od
        0
        >>> del od["Color"]
        >>> od["Color"]
        Traceback (most recent call last):
          File "<stdin>", line 1, in ?
          File "MultiDict.py", line 33, in __getitem__
            return self.data[key]
        KeyError: Color
        >>> list(od.allitems())
        [('Name', 'Andrew'), ('Name', 'Dalke'), (3, 9)]
        >>> list(od2.allkeys())
        ['Name', 'Color', 'Name', 'Color', 3]
        >>> od2["Color"]
        'Green'
        >>> od == od2
        0
        >>>
        >>> s = str(od2)
        >>> s = repr(od2)
    """,
   "test_unordered_multidict": """
        >>> ud = UnorderedMultiDict()
        >>> ud["Name"] = "Andrew"
        >>> ud["Color"] = "BLUE"
        >>> ud["Name"] = "Dalke"
        >>> ud["Color"] = "GREEN"
        >>> ud[3] = 9
        >>> ud[3]
        9
        >>> ud["Name"]
        'Dalke'
        >>> ud["Color"]          # line 11
        'GREEN'
        >>> ud[3]
        9
        >>> len(ud)
        3
        >>> len(list(ud)), len(ud.keys()), len(ud.values()), len(ud.items())
        (3, 3, 3, 3)
        >>> ud["invalid"]
        Traceback (most recent call last):
          File "<stdin>", line 1, in ?
          File "MultiDict.py", line 105, in __getitem__
            return self.data[key][-1]
        KeyError: invalid
        >>> ud.get("invalid")
        >>> ud.get("invalid") is None
        1
        >>> ud.get("invalid", "red")
        'red'
        >>> "Color" in ud
        1
        >>> "Color" in ud.keys()        # line 32
        1
        >>> "invalid" in ud
        0
        >>> "invalid" in ud.keys()
        0
        >>> ud.get("Color", "red")
        'GREEN'
        >>> "Andrew" in ud.values()
        0
        >>> "Dalke" in ud.values()
        1
        >>> ud.getall("Color")           # line 44
        ['BLUE', 'GREEN']
        >>> ud.getall("invalid")
        Traceback (most recent call last):
          File "<stdin>", line 1, in ?
          File "MultiDict.py", line 126, in __getitem__
            return self.data[key]
        KeyError: invalid
        >>> len(list(ud.allkeys())), len(list(ud.allvalues())), len(list(ud.allitems()))
        (5, 5, 5)
        >>> ("Color", "BLUE") in ud.allitems()
        1
        >>> ("Color", "GREEN") in ud.allitems()
        1
        >>> ("Name", "Andrew") in ud.allitems()   # line 58
        1
        >>> ("Name", "Dalke") in ud.allitems()
        1
        >>> (3, 9) in ud.allitems()
        1
        >>> x = list(ud.allkeys())
        >>> x.sort()
        >>> x
        [3, 'Color', 'Color', 'Name', 'Name']
        >>> x = list(ud.allvalues())
        >>> x.sort()
        >>> x
        [9, 'Andrew', 'BLUE', 'Dalke', 'GREEN']
        >>> x = list(ud)
        >>> x.sort()
        >>> x
        [3, 'Color', 'Name']
        >>> ud2 = UnorderedMultiDict(ud)     # line 76
        >>> ud == ud2
        1
        >>> ud != ud
        0
        >>> del ud["Color"]
        >>> ud == ud2
        0
        >>> ud != ud2
        1
        >>> len(ud)
        2
        >>> "Color" in ud
        0
        >>> "Color" in ud2               # line 90
        1
        >>> s = str(ud2)
        >>> s = repr(ud2)
   """,
  "__doc__": __doc__,
}

def _test():
    import doctest, MultiDict
    return doctest.testmod(MultiDict)

if __name__ == "__main__":
    _test()




import unittest
from loggerglue.rfc5424 import *

valids = (
        """<34>1 2003-10-11T22:14:15.003Z mymachine.example.com su - ID47 - \xef\xbb\xbf'su root' failed for lonvick on /dev/pts/8""",
        """<165>1 2003-08-24T05:14:15.000003-07:00 192.0.2.1 myproc 8710 - - %% It's time to make the do-nuts.""",
        """<165>1 2003-10-11T22:14:15.003Z mymachine.example.com evntslog - ID47 [exampleSDID@32473 iut="3" eventSource="Application" eventID="1011"] \xef\xbb\xbfAn application event log entry...""",
        """<165>1 2003-10-11T22:14:15.003Z mymachine.example.com evntslog - ID47 [exampleSDID@32473 iut="3" eventSource="Application" eventID="1011"][examplePriority@32473 class="high"]""",
        """<165>1 2003-10-11T22:14:15.003Z mymachine.example.com evntslog - ID47 [traceback@32473 file="main.py" line="123" method="runStuff" file="pinger.py" line="456" method="pingpong"]""",
        """<34>1 2003-10-11T22:14:15.003000Z mymachine.example.com su - ID47 [test@32473 escaped="\\"nS\\]t\\\u\n"] \xef\xbb\xbf'su root' failed\n for lonvick on /dev/pts/8""",
        """<165>1 2003-10-11T22:14:15.003000Z mymachine.example.com evntslog - ID47 [exampleSDID@32473 iut="3" eventSource="Application" eventID="1011"] \xef\xbb\xbfAn application event log entry...""",
        """<78>1 2011-03-20T12:00:01+01:00 mymachine.example.com - 9778 - - (orion) CMD (/home/www/stats/pinger.py /home/www/stats/data/pinger.pickle)""",
        )

invalids = (
        """This is obviously invalid.""",
        )

class TestABNF(unittest.TestCase):

    def test_valids(self):
        for v in valids:
            r = syslog_msg.parseString(v)

    def test_invalids(self):
        for i in invalids:
            self.assertRaises(ParseException, syslog_msg.parseString, i)

    def test_details(self):
        r = syslog_msg.parseString(valids[0])
        self.assertEqual(r.PRIVAL, '34')
        self.assertEqual(r.VERSION, '1')
        self.assertEqual(r.TIMESTAMP, '2003-10-11T22:14:15.003Z')
        self.assertEqual(r.HOSTNAME, 'mymachine.example.com')
        self.assertEqual(r.APP_NAME, 'su')
        self.assertEqual(r.PROCID, '-')
        self.assertEqual(r.STRUCTURED_DATA, '-')
        self.assertEqual(r.MSG,
                "\xef\xbb\xbf'su root' failed for lonvick on /dev/pts/8")
        r = syslog_msg.parseString(valids[2])
        self.assertTrue(hasattr(r.STRUCTURED_DATA, 'SD_ID'))
        r = syslog_msg.parseString(valids[3])
        self.assertEqual(len(r.SD_ELEMENTS), 2)
        self.assertEqual(len(r.SD_ELEMENTS[0].SD_PARAMS), 3)
        self.assertEqual(len(r.SD_ELEMENTS[1].SD_PARAMS), 1)

class TestSyslogEntry(unittest.TestCase):

    def test_class(self):
        for v in valids:
            se = SyslogEntry.from_line(v)
            self.assertTrue(se is not None)

    def test_details(self):
        se = SyslogEntry.from_line(valids[0])
        self.assertEqual(se.msg,
                """'su root' failed for lonvick on /dev/pts/8""")
        self.assertEqual(se.timestamp.year, 2003)
        self.assertEqual(se.hostname, 'mymachine.example.com')
        self.assertEqual(se.msgid, 'ID47')
        
        se = SyslogEntry.from_line(valids[3])
        self.assertEqual(len(se.structured_data.elements), 2)
        self.assertEqual(len(se.structured_data.elements[0].sd_params), 3)
        self.assertEqual(len(se.structured_data.elements[1].sd_params), 1)

        se = SyslogEntry.from_line(valids[4])
        self.assertEqual(len(se.structured_data.elements), 1)
        self.assertEqual(len(list(se.structured_data.elements[0].sd_params.allitems())), 6)
        self.assertEqual(len(list(se.structured_data.elements[0].sd_params.getall("file"))), 2)
        
        se = SyslogEntry.from_line(valids[5])
        self.assertEqual(str(se), valids[5])
        
        se = SyslogEntry(
                prival=165, version=1, timestamp=datetime(2003,10,11,22,14,15,3000), 
                hostname='mymachine.example.com', app_name='evntslog', procid=None, msgid='ID47',
                structured_data=StructuredData([SDElement('exampleSDID@32473', 
                    [('iut','3'),
                    ('eventSource','Application'),
                    ('eventID','1011')]
                    )]), 
                msg=u'An application event log entry...'
        )
        self.assertEqual(str(se), valids[6])
        
        se = SyslogEntry.from_line(valids[7])
        self.assertEqual(se.timestamp.year, 2011)
        

if __name__ == '__main__':
    unittest.main()

